;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
; Mosaic Thwimp
; by Mandew
;
; Mosaic Thwimp will go a direction until it hits a wall,
; then it will wait,
; then it will turn around and repeat process.
;
; Extra bit: yes
;  If set, goes up and down instead of left and right.
; 
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

; DEFINES

!Speed = #$10		;
!TimeToWait = #$18	; both costumizable

; GRAPHICS

!Tile = #$A2

; SPRITE TABLES

!SpriteYSpeed = $AA
!SpriteXSpeed = $B6
!SpriteBehavior	= $C2
!SpriteStatus = $14C8
!SpriteStunTimer = $1528
!SpriteObjectsStatus = $1588
!SpriteSlopeStatus = $15B8
!SpriteYPosLo = $D8
!SpriteYPosHi = $14D4
!ExtraBits = $7FAB10
!HorizontalDirection = $157C

; ENGINE

!LockFlag = $9D

; ROUTINES
!MarioInteract = $01A7DC
!UpdateYPos = $01801A
!UpdateXPos = $018022
!ObjectsInteract = $019138

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; CODE START
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;


print "INIT ",pc
	LDA #$01
	STA !HorizontalDirection,x
	RTL
print "MAIN ",pc
	PHB
	PHK
	PLB                     
	JSR MainCode 
	PLB
	RTL

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; Main
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
Return:
	LDA #$01                ;x movement related
	JSL $018042	; I don't know what this is for, but it sounds important
	RTS

MainCode:
	JSR DrawGFX

	LDA !SpriteStatus,x
	CMP #$08
	BNE Return
	LDA !LockFlag
	BNE Return

	JSR SubOffscreen0Bnk2
	JSL !MarioInteract
	JSL !UpdateYPos
	JSL !UpdateXPos
	JSL !ObjectsInteract

	LDA !SpriteBehavior,x
	BEQ GoWhicheverWay

	JSR WaitForTime	
	LDA #$01                ;x movement related
	JSL $018042	; I don't know what this is for, but it sounds important
	RTS

GoWhicheverWay:
	LDA !ExtraBits,x
	AND #$04
	BNE UpDown

LeftRight:
	JSR MosaicLR
	LDA #$01                ;x movement related
	JSL $018042	; I don't know what this is for, but it sounds important
	RTS

UpDown:
	JSR MosaicUD
	LDA #$01                ;x movement related
	JSL $018042	; I don't know what this is for, but it sounds important
	RTS

;;;;;;;;;;;;;;;;;;;;;;;;


WaitForTime:
	LDA !SpriteStunTimer,x
	DEC
	STA !SpriteStunTimer,x
	BNE ReturnWait
	STZ !SpriteBehavior,x
	LDA !HorizontalDirection,x
	EOR #$01
	STA !HorizontalDirection,x

ReturnWait:
	RTS


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

LRSpeeds:
	db !Speed,256-!Speed

MosaicLR:
	LDY !HorizontalDirection,x
	LDA.l LRSpeeds,y
	STA !SpriteXSpeed,x

	CPY #$00
	BNE CheckRight

CheckLeft:
	LDA !SpriteObjectsStatus,x
	AND #$01
	BEQ ReturnLR
	BRA StopLR

CheckRight:
	LDA !SpriteObjectsStatus,x
	AND #$02
	BEQ ReturnLR
StopLR:
	JSR StopTime
ReturnLR:
	RTS

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

UDSpeeds:
	db !Speed,256-!Speed

MosaicUD:
	LDY !HorizontalDirection,x
	LDA.l UDSpeeds,y
	STA !SpriteYSpeed,x

	CPY #$00
	BNE CheckDown

CheckUp:
	LDA !SpriteObjectsStatus,x
	AND #$04
	BEQ ReturnUD
	BRA StopUD

CheckDown:
	LDA !SpriteObjectsStatus,x
	AND #$08
	BEQ ReturnUD
StopUD:
	JSR StopTime
ReturnUD:
	RTS

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

StopTime:
	STZ !SpriteXSpeed,x
	STZ !SpriteYSpeed,x
	INC !SpriteBehavior,x
	LDA !TimeToWait
	STA !SpriteStunTimer,x
	RTS

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; GENERIC GRAPHICS ROUTINE
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

DrawGFX:            JSR GET_DRAW_INFO       ; after: Y = index to sprite OAM ($300)
                                            ;      $00 = sprite x position relative to screen boarder 
                                            ;      $01 = sprite y position relative to screen boarder  
                   
                    LDA $00                 ; set x position of the tile
                    STA $0300,y

                    LDA $01                 ; set y position of the tile
                    STA $0301,y

                    LDA #!Tile
                    STA $0302,y

                    LDA $15F6,x             ; get sprite palette info
                    ORA $64                 ; add in the priority bits from the level settings
                    STA $0303,y             ; set properties

                    LDY #$00                ; #$02 means the tiles are 16x16
                    LDA #$00                ; This means we drew one tile
                    JSL $01B7B3
                    RTS

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; Get Draw Info
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

TABLE1:             db $0C,$1C
TABLE2:             db $01,$02

GET_DRAW_INFO:      STZ $186C,x             ; reset sprite offscreen flag, vertical
                    STZ $15A0,x             ; reset sprite offscreen flag, horizontal
                    LDA $E4,x               ; \
                    CMP $1A                 ;  | set horizontal offscreen if necessary
                    LDA $14E0,x             ;  |
                    SBC $1B                 ;  |
                    BEQ ON_SCREEN_X         ;  |
                    INC $15A0,x             ; /

ON_SCREEN_X:        LDA $14E0,x             ; \
                    XBA                     ;  |
                    LDA $E4,x               ;  |
                    REP #$20                ;  |
                    SEC                     ;  |
                    SBC $1A                 ;  | mark sprite invalid if far enough off screen
                    CLC                     ;  |
                    ADC #$0040            ;  |
                    CMP #$0180            ;  |
                    SEP #$20                ;  |
                    ROL A                   ;  |
                    AND #$01                ;  |
                    STA $15C4,x             ; / 
                    BNE INVALID             ; 
                    
                    LDY #$00                ; \ set up loop:
                    LDA $1662,x             ;  | 
                    AND #$20                ;  | if not smushed (1662 & 0x20), go through loop twice
                    BEQ ON_SCREEN_LOOP      ;  | else, go through loop once
                    INY                     ; / 
ON_SCREEN_LOOP:     LDA $D8,x               ; \ 
                    CLC                     ;  | set vertical offscreen if necessary
                    ADC TABLE1,y            ;  |
                    PHP                     ;  |
                    CMP $1C                 ;  | (vert screen boundry)
                    ROL $00                 ;  |
                    PLP                     ;  |
                    LDA $14D4,x             ;  | 
                    ADC #$00                ;  |
                    LSR $00                 ;  |
                    SBC $1D                 ;  |
                    BEQ ON_SCREEN_Y         ;  |
                    LDA $186C,x             ;  | (vert offscreen)
                    ORA TABLE2,y            ;  |
                    STA $186C,x             ;  |
ON_SCREEN_Y:        DEY                     ;  |
                    BPL ON_SCREEN_LOOP      ; /

                    LDY $15EA,x             ; get offset to sprite OAM
                    LDA $E4,x               ; \ 
                    SEC                     ;  | 
                    SBC $1A                 ;  | $00 = sprite x position relative to screen boarder
                    STA $00                 ; / 
                    LDA $D8,x               ; \ 
                    SEC                     ;  | 
                    SBC $1C                 ;  | $01 = sprite y position relative to screen boarder
                    STA $01                 ; / 
                    RTS                     ; return

INVALID:            PLA                     ; \ return from *main gfx routine* subroutine...
                    PLA                     ;  |    ...(not just this subroutine)
                    RTS                     ; /



;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; Sprite Graphics Routine
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

DATA_02D003:        db $40,$B0
DATA_02D005:        db $01,$FF
DATA_02D007:        db $30,$C0,$A0,$C0,$A0,$70,$60,$B0
DATA_02D00F:        db $01,$FF,$01,$FF,$01,$FF,$01,$FF


SubOffscreen0Bnk2:  STZ $03                   ; / 
                    JSR IsSprOffScreenBnk2  ; \ if sprite is not off screen, return 
                    BEQ Return02D090          ; / 
                    LDA $5B     ; \  vertical level 
                    AND #$01                ;  | 
                    BNE VerticalLevelBnk2     ; / 
                    LDA $03                   
                    CMP #$04                
                    BEQ CODE_02D04D           
                    LDA $D8,X       ; \ 
                    CLC                       ;  | 
                    ADC #$50                ;  | if the sprite has gone off the bottom of the level... 
                    LDA $14D4,X     ;  | (if adding 0x50 to the sprite y position would make the high byte >= 2) 
                    ADC #$00                ;  | 
                    CMP #$02                ;  | 
                    BPL OffScrEraseSprBnk2    ; /    ...erase the sprite 
                    LDA $167A,X   ; \ if "process offscreen" flag is set, return 
                    AND #$04                ;  | 
                    BNE Return02D090          ; / 
CODE_02D04D:        LDA $13      
                    AND #$01                
                    ORA $03                   
                    STA $01                   
                    TAY                       
                    LDA $1A    
                    CLC                       
                    ADC DATA_02D007,Y       
                    ROL $00                   
                    CMP $E4,X       
                    PHP                       
                    LDA $1B    
                    LSR $00                   
                    ADC DATA_02D00F,Y       
                    PLP                       
                    SBC $14E0,X     
                    STA $00                   
                    LSR $01                   
                    BCC CODE_02D076           
                    EOR #$80                
                    STA $00                   
CODE_02D076:        LDA $00                   
                    BPL Return02D090          
OffScrEraseSprBnk2: LDA $14C8,X             ; \ If sprite status < 8, permanently erase sprite 
                    CMP #$08                ;  | 
                    BCC OffScrKillSprBnk2     ; / 
                    LDY $161A,X ; \ Branch if should permanently erase sprite 
                    CPY #$FF                ;  | 
                    BEQ OffScrKillSprBnk2     ; / 
                    LDA #$00                ; \ Allow sprite to be reloaded by level loading routine 
                    STA $1938,Y ; / 
OffScrKillSprBnk2:  STZ $14C8,X             ; Erase sprite 
Return02D090:       RTS                       ; Return 

VerticalLevelBnk2:  LDA $167A,X   ; \ If "process offscreen" flag is set, return 
                    AND #$04                ;  | 
                    BNE Return02D090          ; / 
                    LDA $13      ; \ Return every other frame 
                    LSR                       ;  | 
                    BCS Return02D090          ; / 
                    AND #$01                
                    STA $01                   
                    TAY                       
                    LDA $1C    
                    CLC                       
                    ADC DATA_02D003,Y       
                    ROL $00                   
                    CMP $D8,X       
                    PHP                       
                    LDA $1D  
                    LSR $00                   
                    ADC DATA_02D005,Y       
                    PLP                       
                    SBC $14D4,X     
                    STA $00                   
                    LDY $01                   
                    BEQ CODE_02D0C3           
                    EOR #$80                
                    STA $00                   
CODE_02D0C3:        LDA $00                   
                    BPL Return02D090          
                    BMI OffScrEraseSprBnk2    
IsSprOffScreenBnk2: LDA $15A0,X 
                    ORA $186C,X 
                    RTS                       ; Return 